<?php
/*

 A configurable date parser
 
 
 Bugs
 	timezone symbol (eg. EDT) %Z is unsupported yet
 	please report any other bug to author

 @Version    0.3
 
 @Author     Leo West <west_leo AT yahoo.com>
 
 @Notes
	%Z (3 letters timezone) is not supported yet and silently ignored
	
 @Benchmarks
 
	Average parsing time on a P600, Linux 2.4.3, Php 4.0.6 platform.
	Format                              Timing (ms)
	unix timestamp %s                   0.6
	date  %Y %m %d                      1.0
	time  %H:%M:%S	                     1.1
	iso compact  %Y%m%d%H%M%S           1.4
	iso large "%Y-%m-%dT%H:%M:%S        1.5
	RFC822  %a, %d %b %Y %H:%M:%S %Z    1.8
*/

require_once( "Date/Date.php" );
	
class Date_Parser
{
	
	function Date_Parser()
	{
		$this->shortmonths = array( "jan", "feb",  "mar", "apr", "may", "jun", "jul", "aug", "sep", "oct", "nov", "dec" );
		$this->months = array( "january", "february",  "march", "april", "may", "june", "july", "august", "september", "october", "november", "december" );
		$this->weekdays = array( "sun","mon","tue","wen","thu","fri","sat" );
		$this->patterns = array(
			'Y' => '(\d{4})',	// year (yyyy)
			'y' => '(\d{2})',	// year (yy)
			'U' => '(\d{2})',	// week number (00-52)
			'W' => '(\d{2})',	// week number (01-53)
			'm' => '(\d{2})',	// mont (01-12)
			'b' => '(\w{3})',	// mont abrrev. (Jan)
			'h' => '(\w{3})',	// same as b
			'B' => '(\w+)',		// month full name (January)
			'd' => '(\d{2})',	// day of month (01-31)
			'j' => '(\d{3})',	// day of year (001-366)
			'a' => '(\w{3})',	// abbrev. day of week (Sat)
			'w' => '(\d)',		// day of week (0-6)
			'e' => '([ \d]\d)',	// abbrev. space padded day of month ( 1-31)
			'T' => '(\d{2}):(\d{2}):(\d{2})',	// time (hh:mm:ss)
			'H' => '(\d{2})',		// hour (00-23)
			'k' => '(\d{1,2})',	// hour (0-23)
			'I' => '(\d{2})',		// hour (01-12)
			'l' => '(\d{1,2})',	// hour (1-12)
			'M' => '(\d{2})',		// minutes (00-59)
			'S' => '(\d{2})',		// seconds (00-59)
			'p' => '(\w{2})', 	// US time code (AM|PM)
			's' => '(\d+)', 		// Unix epoq 
			'z' => '([+-])(\d{2})(\d{2})', 	// rfc822 timezone (+/-hhmm)
			'Z' => '(\w{3})', 	// 3 chars timezone symbol (EDT)
			'n' => '\r?\n',	// a EOL char
			't' => '\t',	// a Tabulation char
		);
	
	} // Date_Parser
	

/*
 * DateParser::Parse
 * Takes  a date format and a string representinf a date 
 * and return a Date object initialized to this date/time
 * invoke this method in a static way, see example
 * @param string $format the format, same as strftime(3)
 * @param string $string the date as a string
 * @return Date object or false when string is invalid
 * @example
 *  $objDate = DateParser::Parse( "%d %b %Y", "12 Aug 2001" );
 */
	function parse( $format, $string )
	{

		$pos = 0;
		$pattern = "";
		$decode = array();
		// 1. convert the format into a regular expression pattern $pattern
		while( true )
		{
			$c = substr( $format, $pos++, 1 );
			if( $c === false )
				break;
	//			echo "c=$c\n";
			if( $c == '%' ) {
				$code = substr( $format, $pos++, 1 );
				if( $code == '%' ) {
					$pattern .= '%';
					continue;
				}
				$pat = $this->patterns[$code];
				if( ! $pat ) die( "bad code $code" );
				$pattern .= $pat;
				$decode[] = $code;

			} else {
				// TODO pattern escaped
				$pattern .= $c;
			}
		}
		$ret = preg_match( "/^$pattern$/", $string, $regs );
//		echo "preg_match( /^$pattern\$/, $string, $regs ) => $ret\n";
		if( ! $ret ) {
			return false;
		}

		// 2. build the date object
		
		array_shift( $regs );
	/*	
		echo "decode="; print_r( $decode );
		echo "regs="; print_r( $regs );
	*/

		$pos = 0;	
		$date = new Date();
		$date->setTime(0,0,0);

		while( $code = array_shift($decode) ) 
		{
			$value = strtolower($regs[$pos++]);

			switch( $code ) {

			case 'Y':
				if( $value < 1970 || $value > 2038 )
					return $this->Error( "invalid year $val" );
				$date->setYear( $value );
				break;
				
			case 'y':
				$date->setYear( $value + 2000 );
				break;

			case 'm':
				$date->setMonth( $value );
				break;

			case 'd':
			case 'e':
				$date->setDay( $value );
				break;

			case 'j':
				$date->setMonth(1);
				$date->setDay(1);
				$date->addDays( $value-1 );
				break;

			case 'a':
				$wdayidx = array_search( $value, $this->weekdays );
				if( $wdayidx == NULL )
					die( "invalid weekday $value" );
				$date->setWeekDay($wdayidx);
				break;			

			case 'w':
				if( $value < 0 or $value > 6 )
					die( "invalid day of week $value" );
				$date->setWeekDay($value);
				break;

			case 'H':
			case 'I':
			case 'k':
			case 'l':
				$date->setHours( $value );
				break;

			case 'M':
				$date->setMinutes( $value );
				break;

			case 'S':
				$date->setSeconds( $value );
				break;

			case 'p':
				if( $value == "pm" )
					$date->addHours( 12 );
				elseif( $value != "am" )
					die( "$value is not a valid time of day code (AM/PM)" );
				break;
			case 'T':
				array_unshift( $decode, 'H', 'M', 'S' );
				$pos--;
				break;

			case "b":
			case "h":
				$bpos = array_search( $value, $this->shortmonths );
				if( $bpos === false )
					return $this->Error( "invalid month shortname $value" );
				$date->setMonth( $bpos+1 );
				break;
			case "B":
				$monthidx = array_search( $value, $this->months );
				if( $monthidx === NULL )
					die( "invalid month name $val" );
				$date->setMonth( $monthidx+1 );
				break;

			case 's':
				$date->setTimestamp( $value );
				break;

			case 'z':
				// we want time in UTC
				if( $value == "+" ) {
					$date->addHours( -$regs[$pos++] );
					$date->addMinutes( -$regs[$pos++] );
				} else {
					$date->addHours( $regs[$pos++] );
					$date->addMinutes( $regs[$pos++] );
				}
				break;

			case 'Z':
				// @TODO : lookup in a timezone map, need the table of codes
				// $date->addHours( $dH );	$date->addMinutes( $dM );
				die( "%Z is unsupported yet" );
				break;

			case 'n':
			case 't':
				break;
			default:
				die( "sorry, code $code not fully handled yet" );
			}
		}
		print_r($date);				
		if( ! $date->isValid() )
			return false;
		else
			return $date;
	}

	function Error( $msg )
	{
		$this->errorMsg = $msg;
		return false;
	}
	
	
} //class Date_Parser

?>
